"""
.. codeauthor:: Nathan Hertz <nhertz@nrao.edu>

File containing definitions for one half of the capability side of the Workspaces REST API,
concerning capabilities themselves
"""

from pyramid.httpexceptions import (
    HTTPBadRequest,
    HTTPExpectationFailed,
    HTTPNotFound,
    HTTPPreconditionFailed,
)
from pyramid.request import Request
from pyramid.response import Response
from pyramid.view import view_config

from workspaces.capability.schema import Capability


@view_config(route_name="view_capability", renderer="json")
def view_capability(request: Request) -> Response:
    """
    Pyramid view that accepts a request to view a capability and responds with the requested capability's
    info, if it exists
    URL: capability/{capability_name}

    :param request: GET request
    :return: Response with JSON-formatted capability info if found
        or 404 response (HTTPNotFound)
    """
    capability = request.capability_info.lookup_capability(request.matchdict["capability_name"])
    if capability:
        return Response(json_body=capability.__json__())
    else:
        return HTTPNotFound(detail=f"Capability {request.matchdict['capability_name']} not found.")


@view_config(route_name="create_capability", renderer="json")
def create_capability(request: Request) -> Response:
    """
    Pyramid view that accepts a request to create a capability
    URL: capability/create

    :param request: POST request, expecting JSON parameters ["capability_name", "steps", "max_jobs"]
        optionally, accepts a boolean "enabled" parameter
    :return: Response with JSON-formatted capability info of newly created capability
        or 400 response (HTTPBadRequest) if expected parameters not given
        or 412 response (HTTPPreconditionFailed) if capability with given name already exists
    """
    expected_params = ["capability_name", "steps", "max_jobs", "has_image_products"]
    params = request.json_body

    if not all([expected in params for expected in expected_params]):
        # JSON params do not contain all expected params
        params_not_given_msg = (
            f"Expected JSON parameters {expected_params}. Received only {params}."
        )
        return HTTPBadRequest(detail=params_not_given_msg)
    elif request.capability_info.lookup_capability(params["capability_name"]):
        # Capability with given name already exists
        already_exists_msg = (
            f"Capability {params['capability_name']} already exists.",
            f"To instead edit existing capability, use capability/{params['capability_name']}/edit.",
        )
        return HTTPPreconditionFailed(detail=already_exists_msg)
    else:
        new_capability = Capability(
            name=params["capability_name"],
            steps=params["steps"],
            max_jobs=params["max_jobs"],
            enabled=params.get("enabled", True),
            has_image_products=params.get("has_image_products", False),
        )
        request.capability_info.save_entity(new_capability)
        return Response(json_body=new_capability.__json__())


@view_config(route_name="edit_capability", renderer="json")
def edit_capability(request: Request) -> Response:
    """
    Pyramid view that accepts a request to edit a capability
    URL: capability/{capability_name}/edit

    :param request: POST request, expecting JSON parameters ["capability_name", "steps", "max_jobs"]
    :return: Response with JSON-formatted capability info of newly created capability
        or 400 response (HTTPBadRequest) if expected parameters not given
        or 412 response (HTTPPreconditionFailed) if capability with given name does not exist
        or 417 response (HTTPExpectationFailed) if the capability was unable to be edited

    TODO: In the future, we should check if there are any active requests for the capability being edited and
        disallow its editing until they are finished
    """
    expected_params = ["steps", "max_jobs"]
    capability_name = request.matchdict["capability_name"]
    params = request.json_body

    if not any([expected in params for expected in expected_params]):
        # JSON params do not contain any expected params
        params_not_given_msg = (
            f"Expected one or more JSON parameters {expected_params}. Received only {params}."
        )
        return HTTPBadRequest(detail=params_not_given_msg)
    elif not request.capability_info.lookup_capability(capability_name):
        # Capability with given name does not exist
        does_not_exist_msg = (
            f"Capability {capability_name} does not exist.",
            f"To instead create a new capability, use capability/create with the same parameters.",
        )
        return HTTPPreconditionFailed(detail=does_not_exist_msg)
    else:
        steps = params.get("steps", None)
        max_jobs = params.get("max_jobs", None)
        success = request.capability_info.edit_capability(capability_name, steps, max_jobs)
        if success:
            return Response(body="Capability successfully edited!")
        else:
            return HTTPExpectationFailed(detail=f"Unable to edit capability {capability_name}.")


@view_config(route_name="enable_capability", renderer="json")
def enable_capability(request: Request) -> Response:
    """
    Pyramid view that enables a capability

    :param request: POST request
    :return: HTTP 200 response
        or 412 response (HTTPPreconditionFailed) if capability with given name does not exist
        or 417 response (HTTPExpectationFailed) if the capability was unable to be enabled
    """
    capability_name = request.matchdict["capability_name"]

    if not request.capability_info.lookup_capability(capability_name):
        # Capability with given name does not exist
        does_not_exist_msg = f"Capability {capability_name} does not exist. Cannot enable."
        return HTTPPreconditionFailed(detail=does_not_exist_msg)
    else:
        success = request.capability_info.edit_capability(capability_name, enabled=True)
        if success:
            return Response(body="Capability successfully enabled!")
        else:
            return HTTPExpectationFailed(detail=f"Unable to enable capability {capability_name}.")


@view_config(route_name="disable_capability", renderer="json")
def disable_capability(request: Request) -> Response:
    """
    Pyramid view that disables a capability

    :param request: POST request
    :return: HTTP 200 response
        or 412 response (HTTPPreconditionFailed) if capability with given name does not exist
        or 417 response (HTTPExpectationFailed) if the capability was unable to be disabled
    """
    capability_name = request.matchdict["capability_name"]

    if not request.capability_info.lookup_capability(capability_name):
        # Capability with given name does not exist
        does_not_exist_msg = f"Capability {capability_name} does not exist. Cannot disable."
        return HTTPPreconditionFailed(detail=does_not_exist_msg)
    else:
        success = request.capability_info.edit_capability(capability_name, enabled=False)
        if success:
            return Response(body="Capability successfully disabled!")
        else:
            return HTTPExpectationFailed(detail=f"Unable to disable capability {capability_name}.")
